import org.apache.tools.ant.taskdefs.Java;
import java.text.SimpleDateFormat;

void usage() {
        println "-------------------------------------------------------------";
	println "Script to smoke test JBDS installation";
	println "usage: groovy.sh smokeJBDS.groovy <jbds_home> | <jbds.config.file>";
	println "       <jbds_home>: JBDS smoke tests will be executed against eclipse/jbds instance";
        println "--------------------------------------------------------------";
}


void invokeTests(File eclipseHome, String productName, String additionalVMArgs, String testClassName) {
	String report = "TEST-smoke-" + new SimpleDateFormat("yyyyMMddh-hmm").format(new Date()) + ".xml";

	String specificVMArgs="";
	String osName = System.properties['os.name'].toLowerCase();
	if(osName.contains("mac")){
		specificVMArgs="-XstartOnFirstThread";
	}
	// Invoke tests
	Java proc = new org.apache.tools.ant.taskdefs.Java();
	proc.setFork(true);
	proc.setCloneVm(true);
	proc.setDir(eclipseHome);
	proc.setJvmargs(additionalVMArgs + " " +
			specificVMArgs   + " " +
			"-Dorg.eclipse.swtbot.search.timeout=30000 " +
			"-Dusage_reporting_enabled=false " +
			"-Xms256M -Xmx768M -XX:MaxPermSize=512M");
	proc.setJar(new File(eclipseHome, "plugins").listFiles().find {it.getName().startsWith("org.eclipse.equinox.launcher_") && it.getName().endsWith(".jar")} );
	proc.setArgs("-application org.eclipse.swtbot.eclipse.junit.headless.swtbottestapplication " +
			"-testApplication org.eclipse.ui.ide.workbench " +
			"-product " + productName + " " +
			"-data workspace " +
			"formatter=org.apache.tools.ant.taskdefs.optional.junit.XMLJUnitResultFormatter," + report + " " +
			"formatter=org.apache.tools.ant.taskdefs.optional.junit.PlainJUnitResultFormatter " +
			"-testPluginName com.jboss.jbds.smoke.test " +
			"-className " + testClassName + " " +
			"-consoleLog -debug");
	proc.init();
	int returnCode = proc.executeJava();
	if (returnCode != 0) {
		println("An error occured. Most probably because of wrong configuration of environment.");
		System.exit(1);
	}

	File report_file = new File(eclipseHome.getAbsolutePath() + "/" + report);

	report_file.eachLine { line ->
		if (line.contains("failures")) {
			if (line.contains("errors=\"0\" failures=\"0\"")) {
				println("Install SUCCESS. Read " + report + " for more details.");
				return;
			} else {
				println("Failed to install. Read " + report + " for details and see screenshots/");
				System.exit(1);
			}
		}
	}
}

String getTestSuite(){
	String installEAP = System.properties['jbds.installer.eap.install'];

	if(System.properties['testSuite'] != null){
		return System.properties['testSuite'];
	}
	//return defaults
	if(installEAP != null && installEAP.equals("true")){

		//default for jbds with embedded eap is
		return "com.jboss.jbds.smoke.test.suite.SmokeAllTest";

	}else {

		//default for jbds without EAP
		return "com.jboss.jbds.smoke.test.suite.SmokeAllNonEAPTest";
	}
}


if (args.length < 1) {
	usage();
	System.exit(2);
}

Properties properties = new Properties();
File eclipseHome;
File inputArg = new File(args[0]);


if(inputArg.isDirectory()){
	
	eclipseHome = new File(args[0] + "/studio");
	println("jbds_home:" + eclipseHome);

}else if(inputArg.isFile()){

	//read properties from file
	FileInputStream inp = new FileInputStream(args[0]);
	properties.load(inp);
	inp.close();

        /* export properties loaded from file to System properties,
           they will be used by test plugin */
        for (String name : properties.stringPropertyNames()) {
            String value = properties.getProperty(name);
            System.setProperty(name, value);
        }

    
	String jbds_folder = properties['jbds.installer.target.folder'];
	if(jbds_folder != null){
		eclipseHome = new File(jbds_folder + "/studio");
		println("jbds_home:" + eclipseHome);
	}else{
		println("jbds.installer.target.folder property not found in " + args[0] + " config file!");
		System.exit(2);
	}
}else{
	usage();
	System.exit(2);
}



println "Preparing tests, installing framework";

String companionRepoLocation = System.getProperty("companionRepo");
if (companionRepoLocation == null) {
	companionRepoLocation = getClass().protectionDomain.codeSource.location.path;
	companionRepoLocation = companionRepoLocation[0..companionRepoLocation.lastIndexOf('/')];
	companionRepoLocation += "repository";
	println companionRepoLocation;
}
// Install test framework
Java proc = new org.apache.tools.ant.taskdefs.Java();
proc.setDir(eclipseHome);
proc.setFork(true);
proc.setJar(new File(eclipseHome, "plugins").listFiles().find({it.getName().startsWith("org.eclipse.equinox.launcher_") && it.getName().endsWith(".jar")}).getAbsoluteFile());
proc.setArgs("-application org.eclipse.equinox.p2.director " +
		"-repository http://download.eclipse.org/technology/swtbot/releases/latest/," +
		"file:///" + companionRepoLocation + " " +
		"-installIU com.jboss.jbds.smoke.test " +
		"-installIU org.eclipse.swtbot.eclipse.test.junit.feature.group " +
		"-consolelog");
proc.init();
int returnCode = proc.executeJava();
if (returnCode != 0) {
	System.exit(3);
}

File iniFile;
String osName = System.properties['os.name'].toLowerCase();
if(osName.contains("mac")){
	//Mac OSX
	iniFile = new File(eclipseHome.getAbsolutePath() + "/Eclipse.app/Contents/MacOS").listFiles().find({it.getName().endsWith(".ini")});
	if(iniFile == null){
	iniFile = new File(eclipseHome.getAbsolutePath() + "/JBoss Developer Studio.app/Contents/MacOS").listFiles().find({it.getName().endsWith(".ini")});
	}
}else{
	iniFile = eclipseHome.listFiles().find({it.getName().endsWith(".ini")});
}
iniLines = iniFile.readLines();
targetIndex = iniLines.findIndexOf {line -> line.startsWith("-product") };
String productName = iniLines[targetIndex + 1];
println ("Product is: " + productName);

//Invoke Tests here
println "Invoking tests: " + getTestSuite();

invokeTests(eclipseHome, productName, "", getTestSuite());


System.exit(0);
